﻿using Microsoft.Identity.Client;
using Microsoft.PowerBI.Api;
using Microsoft.Rest;
using System.Linq;
using System;
using Microsoft.PowerBI.Api.Models;
using System.Collections.Generic;

namespace DemoPowerBI
{
    class RealtimeLog
    {
        public RealtimeLog() { }

        public DateTime Date { get; set; }
        public DateTime Time { get; set; }
        public string Client { get; set; }
        public double Duration { get; set; }
        public double Amount { get; set; }
    }

    class Program
    {
        static PowerBIClient _powerBIClient = null;

        #region Authentication settings
        // Authentication settings
        private const string clientId = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx";   // Created in AD (Realtime Demo)
        private const string clientSecret = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"; // Application client secret (generated in AAD for RealtimeDemo app)
        private const string tenantId = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx";   // AAD Tenant ID (contoso-bi)
        private static readonly string authority = $"https://login.microsoftonline.com/{tenantId}";
        #endregion

        // Power BI API settings
        private const string resource = "https://analysis.windows.net/powerbi/api";
        private const string ApiUrl = "https://api.powerbi.com";

        // Dataset and Group settings
        private const string RealtimeDatasetName = "Realtime Dataset";  // You can change this name pointing to the Dataset you want to create/update

        // Retrieve this value from the Workspace group ID
        // This can be easily retrieved from the URL when you navigate on powerbi.com
        // and it is the string just after "groups" in the URL, for example:
        // https://app.powerbi.com/groups/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        static readonly Guid GroupRealtimeDemoId = new Guid("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"); // This is the "Realtime Demos" workspace name in Power BI
                                                                                                     

        static string _powerbi_datasetId_RT = null;

        /// <summary>
        /// This initialization works with the service provider
        /// </summary>
        static void InitPowerBI()
        {
            #region CreateAccessToken
            IConfidentialClientApplication app = ConfidentialClientApplicationBuilder
                .Create(clientId)
               .WithClientSecret(clientSecret)
               .WithAuthority(new Uri(authority))
               .Build();

            // Use .default to get all the permissions available (those configured in AAD for this app)
            string[] scopes = new string[] { $"{resource}/.default" };

            AuthenticationResult result = null;
            try
            {
                result = app.AcquireTokenForClient(scopes).ExecuteAsync().Result;
            }
            catch (MsalUiRequiredException ex)
            {
                // The application doesn't have sufficient permissions.
                // - Did you declare enough app permissions during app creation?
                // - Did the tenant admin grant permissions to the application?
                Console.WriteLine(ex.Message);
            }
            catch (MsalServiceException ex) when (ex.Message.Contains("AADSTS70011"))
            {
                // Invalid scope. The scope has to be in the form "https://resourceurl/.default"
                // Mitigation: Change the scope to be as expected.
                Console.WriteLine(ex.Message);
            }

            #endregion

            Console.WriteLine($"App-Only Access Token:\n{result.AccessToken}\n");
            var tokenCredentials = new TokenCredentials(result.AccessToken, "Bearer");
            _powerBIClient = new PowerBIClient(new Uri(ApiUrl), tokenCredentials);
        }

        static Random _random = new Random();

        private static void LogPowerBI_RT(PowerBIClient api, string DatasetName)
        {
            RealtimeLog[] logItems = new RealtimeLog[5000];
            List<object> rows = new List<object>();

            int rowsExpected = _random.Next(3, 10);
            for (int i = 0; i < rowsExpected; i++)
            {
                DateTime timestamp = DateTime.UtcNow;
                var daxLog = new RealtimeLog
                {
                    Date = timestamp.Date,
                    Time = new DateTime(1990, 1, 1, timestamp.Hour, timestamp.Minute, timestamp.Second),
                    Client = Environment.MachineName,
                    Duration = _random.NextDouble() * 1000,
                    Amount = 1000 + _random.NextDouble() * 10000
                };
                rows.Add(daxLog);
            }

            // Retrieve the dataset ID and create it if it doesn't exist
            if (string.IsNullOrEmpty(_powerbi_datasetId_RT))
            {
                var datasets = api.Datasets.GetDatasetsInGroup(GroupRealtimeDemoId).Value;
                var ds = datasets.Where(d => d.Name == DatasetName).FirstOrDefault();
                if (ds == null)
                {
                    // Create the schema
                    var tables = new Type[] { typeof(RealtimeLog) };
                    var schema = SchemaBuilder.GetDataset(DatasetName, ref tables);
                    ds = api.Datasets.PostDatasetInGroup(GroupRealtimeDemoId, schema, DefaultRetentionPolicy.BasicFIFO);
                }
                _powerbi_datasetId_RT = ds.Id;
            }

            api.Datasets.PostRowsInGroup(GroupRealtimeDemoId, _powerbi_datasetId_RT, typeof(RealtimeLog).Name, new PostRowsRequest(rows));
        }

        static void Main(string[] args)
        {
            // Initialize API and authenticate
            InitPowerBI();

            // Push data
            LogPowerBI_RT(_powerBIClient, RealtimeDatasetName);

            Console.WriteLine("Done!");
        }
    }
}
